# cta.py

import csv

# Cache of frequently used strings

_cache = { }

def cached(s):
    if s in _cache:
        return _cache[s]
    else:
        _cache[s] = s
        return s
    
class Row:
    __slots__ = ('route', 'date', 'daytype', 'rides')
    def __init__(self, route, date, daytype, rides):
        self.route = route
        self.date = date
        self.daytype = daytype
        self.rides = rides
        
def read_rides(filename):
    records = []
    with open(filename) as file:
        rows = csv.reader(file)
        headers = next(rows)
        for row in rows:
            record = Row(cached(row[0]), cached(row[1]), row[2], int(row[3]))
            records.append(record)
    return records

records = read_rides('ctabus.csv')

# Option 1: Nothing (leave "as is")
# 213 MB

# Option 2: Tuples (with some data conversion)
# 183 MB

# Option 3: Dicts
# 300 MB

# Option 4: Classes
# 247MB
# With slots:  177MB

# Option 5 : Pandas
# 58MB

import os
print("PID", os.getpid())

# Problem 1: How many bus routes in Chicago?
# Uniqueness. Use a set

routes = set()
for record in records:
    routes.add(record.route)
print(len(routes), 'routes')

# Problem 2: How many people rode route 22 on Apr 9, 2007
# More generic: Any route, any date?
# Use a dict with composite keys (route, date)

route_date = { }
for record in records:
    route_date[record.route, record.date] = record.rides

print(route_date['22','04/09/2007'])

# Problem 3: 10 most popular bus routes?
# Solution: Use a collections.Counter

from collections import Counter
totals = Counter()
for record in records:
    totals[record.route] += record.rides

print('Most common')
for route, total in totals.most_common(10):
    print(route, total)
    
# Problem 4: 10 most populars routes in 2016?
# Cluster the data by year, then build counters
from collections import defaultdict
by_year = defaultdict(Counter)

for record in records:
    year = record.date[-4:]   # Hack (last 4 digits)
    by_year[year][record.route] += record.rides

# Most popular in 2016
print('Most common 2016')
for route, total in by_year['2016'].most_common(10):
    print(route, total)

# Problem 5 : Greatest increase 2001-2016
diff = by_year['2016'] - by_year['2001']
print("Greatest increase")
for route, delta in diff.most_common(10):
    print(route, delta)
    

